function processAndParseData(fileName,gaitParseLeg,stimSide)

% Load the file
load(fileName);


%% Extract needed variables
markerData = Markers; %rawData.video.markers; %NOTE: This was changed to processed marker data to avoid marker dropping
markerNames = convertCharMatrixToCells(MarkerID); %convertCharMatrixToCells(rawData.video.markersid);
markerSampleRate = VideoFrameRate; %rawData.video.samplerate;

analogSampleRate = rawData.analog.samplerate;
forcePlateData = rawData.analog.plateforces;
forcePlateNames = rawData.analog.plateforcesid;

COPData = rawData.analog.cop;
COPNames = rawData.analog.copid;

EMGData = rawData.analog.emg;
EMGNames = convertCharMatrixToCells(rawData.analog.emgid);

nMarkerFrames = size(markerData,1);
nAnalogFrames = size(forcePlateData,1);
timeMarker = linspace(0,nMarkerFrames/markerSampleRate,nMarkerFrames);
timeAnalog = linspace(0,nAnalogFrames/analogSampleRate,nAnalogFrames);


%% Change units, reorder, and filter data
% Change data to desired units
% Lengths were given in mm, we want m. Similarly moments were given in N-mm
markerData = markerData/1000;
COPData = COPData/1000;
forcePlateData(:,[4,5,6,10,11,12]) = forcePlateData(:,[4,5,6,10,11,12])/1000;


% Reorder data (change axis)
newAxisOrder = [2 3 1]; %[2 3 1] means that the new [1 2 3] axis are the old [2 3 1] axis
markerData_reordered = markerData(:,:,newAxisOrder);
forcePlateData_reordered = forcePlateData(:,[newAxisOrder,newAxisOrder+3,newAxisOrder+6,newAxisOrder+9]);
COPData_reordered = COPData(:,[newAxisOrder,newAxisOrder+3]);

% TO DO: Change order of EMG signals to match up with what Andrew had, then
% remove the order change from "RemoveOutlierTrialsFromData" and
% "MuscleModelOptimization3D".

% Rename markers
newMarkerNames = renameMarkers(markerNames);
% TO DO: Find marker drops and interpolate
% markerData = splineDroppedMarkers(markerData);


%% Determine stimulation on-off times and process EMG data
isStimRight = strcmpi(stimSide,'right');
isStimLeft = strcmpi(stimSide,'left');
isStim = isStimRight || isStimLeft;

if isStim
    % Find which emg signals are for the stimulated side so that we can
    % give just the stimulated side to the getStimIndices function
    stimSideEMGIndices = zeros(1,14);
    stimSideEMGCount = 0;
    if isStimRight
        for iii = 1:length(EMGNames)
            if strcmp(EMGNames{iii}(end),'R')
                stimSideEMGCount = stimSideEMGCount + 1;
                stimSideEMGIndices(stimSideEMGCount) = iii;
            end
        end
    else
        for iii = 1:length(EMGNames)
            if strcmp(EMGNames{iii}(end),'L')
                stimSideEMGCount = stimSideEMGCount + 1;
                stimSideEMGIndices(stimSideEMGCount) = iii;
            end
        end
    end
    if stimSideEMGCount == 0;
        if isStimRight
            stimSideEMGIndices = [1:8,21:26];
        else
            stimSideEMGIndices = [11:18,31:36];
        end
    end
    % Find indices of on-off stimulation 
    stimIndices = getStimIndices(EMGData(:,stimSideEMGIndices),analogSampleRate);
    
    % Set EMGData to zero during stimulation
    stimOnTimes = timeAnalog(stimIndices(1:2:length(stimIndices)-1));
    stimOffTimes = timeAnalog(stimIndices(2:2:length(stimIndices)));
    
    stimRanges = stimIndices(1):stimIndices(2);
    for iii = 3:2:(length(stimIndices)-1);
        stimRanges = [stimRanges,stimIndices(iii):stimIndices(iii+1)];
    end
    EMGData_zeroed = EMGData;
    EMGData_zeroed(stimRanges,stimSideEMGIndices) = 0;
    
    emgData_processed = processEMG(EMGData_zeroed,1/analogSampleRate,30,4);
    
else
    emgData_processed = processEMG(EMGData,1/analogSampleRate,30,4);
end



%% Find heel-strike and toe-off locations and filter data
isGaitParseRight = strcmpi(gaitParseLeg,'right');
isGaitParseLeft = strcmpi(gaitParseLeg,'left');
isGait = isGaitParseRight || isGaitParseLeft;

if isGait
    % Find vertical ground reaction force from limb being used
    GRFVert_right = filterData(forcePlateData_reordered(:,8),1/analogSampleRate,4,4); % was 18 but that makes cycle look parsed wrong because it is re-filtered after parsing
    GRFVert_left = filterData(forcePlateData_reordered(:,2),1/analogSampleRate,4,4);
    
    [cycleBegins_right,cycleEnds_right,heelStrikeFrames_right,toeOffFrames_right] = parseCycles(GRFVert_right);
    [cycleBegins_left,cycleEnds_left,heelStrikeFrames_left,toeOffFrames_left] = parseCycles(GRFVert_left);
    
    if isGaitParseRight
        cycleBegins = cycleBegins_right;
        cycleEnds = cycleEnds_right;
        heelStrikeFrames = heelStrikeFrames_right;
        toeOffFrames = toeOffFrames_right;
    else
        cycleBegins = cycleBegins_left;
        cycleEnds = cycleEnds_left;
        heelStrikeFrames = heelStrikeFrames_left;
        toeOffFrames = toeOffFrames_left;
    end
    numSteps = length(heelStrikeFrames)-1;
    
    % Get which frames are in and out of contact
    [inContactFrames_left,outOfContactFrames_left,inContactFrames_right,outOfContactFrames_right] = ...
        getInOutOfContactFrames(heelStrikeFrames_left,toeOffFrames_left,heelStrikeFrames_right,toeOffFrames_right,nAnalogFrames);
    
    % Zero ground reactions from out of contact points
    forcePlateData_reordered(outOfContactFrames_left,1:6) = 0;
    forcePlateData_reordered(outOfContactFrames_right,7:12) = 0;
    COPData_reordered(outOfContactFrames_left,1:3) = ones(length(outOfContactFrames_left),1)*median(COPData_reordered(inContactFrames_left,1:3));
    COPData_reordered(outOfContactFrames_right,4:6) = ones(length(outOfContactFrames_right),1)*median(COPData_reordered(inContactFrames_right,1:3));
    % Set negative vertical ground reactions to zero
    isGRFVertNegative_left = forcePlateData_reordered(:,2) < 0;
    isGRFVertNegative_right = forcePlateData_reordered(:,8) < 0;
    forcePlateData_reordered(isGRFVertNegative_left,2) = 0;
    forcePlateData_reordered(isGRFVertNegative_right,8) = 0;
    
    % Filter data
    cyclePeriod = (timeAnalog(heelStrikeFrames(end)) - timeAnalog(heelStrikeFrames(1)))/numSteps; 
%     forcePlateData_filtered = zeros(size(forcePlateData_reordered));
%     forcePlateData_filtered(:,[5,11]) = filterData(forcePlateData_reordered(:,[5,11]),1/analogSampleRate,10/cyclePeriod,4);
%     forcePlateData_filtered(:,[3,9]) = filterData(forcePlateData_reordered(:,[3,9]),1/analogSampleRate,14/cyclePeriod,4);
%     forcePlateData_filtered(:,[1:2,4,6:8,10,12]) = filterData(forcePlateData_reordered(:,[1:2,4,6:8,10,12]),1/analogSampleRate,20/cyclePeriod,4);
    forcePlateData_filtered = zeros(size(forcePlateData_reordered));
    forcePlateData_filtered(:,[5,11]) = filterData(forcePlateData_reordered(:,[5,11]),1/analogSampleRate,7/cyclePeriod,4);
    forcePlateData_filtered(:,[3,9]) = filterData(forcePlateData_reordered(:,[3,9]),1/analogSampleRate,7/cyclePeriod,4);
    forcePlateData_filtered(:,[1:2,4,6:8,10,12]) = filterData(forcePlateData_reordered(:,[1:2,4,6:8,10,12]),1/analogSampleRate,7/cyclePeriod,4);
    copData_filtered = filterData(COPData_reordered,1/analogSampleRate,7/cyclePeriod,4);
    
    
    sizeOfMarkerData = size(markerData);
    markerData_filtered = zeros(sizeOfMarkerData);
    for iii = 1:sizeOfMarkerData(2)
        markerData_filtered(:,iii,:) = filterData(squeeze(markerData_reordered(:,iii,:)),1/markerSampleRate,6,2);
    end
    
else
    % Filter data
    forcePlateData_filtered = zeros(size(forcePlateData_reordered));
%     forcePlateData_filtered(:,[5,11]) = filterData(forcePlateData_reordered(:,[5,11]),1/analogSampleRate,10,4);
%     forcePlateData_filtered(:,[3,9]) = filterData(forcePlateData_reordered(:,[3,9]),1/analogSampleRate,14,4);
%     forcePlateData_filtered(:,[1:2,4,6:8,10,12]) = filterData(forcePlateData_reordered(:,[1:2,4,6:8,10,12]),1/analogSampleRate,20,4);
%     copData_filtered = filterData(COPData_reordered,1/analogSampleRate,20,4);
 
% NOTE (Nathan): I changed the GRF filtering cutoff frequency for non-gait
% data from 7 to 4 because the ID loads for FES calibration were very bumpy
    forcePlateData_filtered(:,[5,11]) = filterData(forcePlateData_reordered(:,[5,11]),1/analogSampleRate,4,4);
    forcePlateData_filtered(:,[3,9]) = filterData(forcePlateData_reordered(:,[3,9]),1/analogSampleRate,4,4);
    forcePlateData_filtered(:,[1:2,4,6:8,10,12]) = filterData(forcePlateData_reordered(:,[1:2,4,6:8,10,12]),1/analogSampleRate,4,4);
    copData_filtered = filterData(COPData_reordered,1/analogSampleRate,4,4);
    
    sizeOfMarkerData = size(markerData);
    markerData_filtered = zeros(sizeOfMarkerData);
    for iii = 1:sizeOfMarkerData(2)
        markerData_filtered(:,iii,:) = filterData(squeeze(markerData_reordered(:,iii,:)),1/markerSampleRate,6,2);
    end
    
end


%% Calculate EC, check COP, set negative forces to zero
% Set negative vertical forces to zero
isGRFVertNegative_left = forcePlateData_filtered(:,2) < 0;
isGRFVertNegative_right = forcePlateData_filtered(:,8) < 0;
forcePlateData_filtered(isGRFVertNegative_left,2) = 0;
forcePlateData_filtered(isGRFVertNegative_right,8) = 0;

% Calculate electrical center locations
loadsForECCalc = forcePlateData_filtered;
isLeftFySmall = abs(loadsForECCalc(:,2)) < 1;
isRightFySmall = abs(loadsForECCalc(:,8)) < 1;
loadsForECCalc(isLeftFySmall,2) = 1;
loadsForECCalc(isRightFySmall,8) = 1;
zerosCol = zeros(size(copData_filtered,1),1);
% ECx_left = zerosCol;
% ECz_left = zerosCol;
% ECx_right = zerosCol;
% ECz_right = zerosCol;

h = 0.015; % h is the height offset from the treadmill manual
% Left EC: x = 0.739, z = 0.1144
% Right EC: x = 0.739, z = 0.6914
ECx_left = 0.739;
ECy_left = 0;
ECz_left = 0.1144;
ECx_right = 0.739;
ECy_right = 0;
ECz_right = 0.6914;
% ECx_left = 0.153;
% ECy_left = 0;
% ECz_left = -0.197;
% ECx_right = 0.153;
% ECy_right = 0;
% ECz_right = 0.985;
ECs = [ECx_left,ECy_left,ECz_left,ECx_right,ECy_right,ECz_right];

ECs_time = ones(size(forcePlateData_filtered,1),1)*ECs;

% if isGait
% %     ECx_left(inContactFrames_left) = copData_filtered(inContactFrames_left,1) - ...
% %         (loadsForECCalc(inContactFrames_left,6))./loadsForECCalc(inContactFrames_left,2);
% %     ECx_left(outOfContactFrames_left) = median(ECx_left(inContactFrames_left));
% %     ECy_left = zerosCol;
% %     ECz_left(inContactFrames_left) = copData_filtered(inContactFrames_left,3) - ...
% %         (-loadsForECCalc(inContactFrames_left,4))./loadsForECCalc(inContactFrames_left,2);
% %     ECz_left(outOfContactFrames_left) = median(ECz_left(inContactFrames_left));
% %     
% %     ECx_right(inContactFrames_right) = copData_filtered(inContactFrames_right,4) - ...
% %         (loadsForECCalc(inContactFrames_right,12))./loadsForECCalc(inContactFrames_right,8);
% %     ECx_right(outOfContactFrames_right) = median(ECx_right(inContactFrames_right));
% %     ECy_right = zerosCol;
% %     ECz_right(inContactFrames_right) = copData_filtered(inContactFrames_right,6) - ...
% %         (-loadsForECCalc(inContactFrames_right,10))./loadsForECCalc(inContactFrames_right,8);
% %     ECz_right(outOfContactFrames_right) = median(ECz_right(outOfContactFrames_right));
%     ECx_left(inContactFrames_left) = copData_filtered(inContactFrames_left,1) - ...
%         (h*loadsForECCalc(inContactFrames_left,1) + loadsForECCalc(inContactFrames_left,6))./loadsForECCalc(inContactFrames_left,2);
%     ECx_left(outOfContactFrames_left) = median(ECx_left(inContactFrames_left));
%     ECy_left = zerosCol;
%     ECz_left(inContactFrames_left) = copData_filtered(inContactFrames_left,3) - ...
%         (-h*loadsForECCalc(inContactFrames_left,3) - loadsForECCalc(inContactFrames_left,4))./loadsForECCalc(inContactFrames_left,2);
%     ECz_left(outOfContactFrames_left) = median(ECz_left(inContactFrames_left));
%     
%     ECx_right(inContactFrames_right) = copData_filtered(inContactFrames_right,4) - ...
%         (h*loadsForECCalc(inContactFrames_right,7) + loadsForECCalc(inContactFrames_right,12))./loadsForECCalc(inContactFrames_right,8);
%     ECx_right(outOfContactFrames_right) = median(ECx_right(inContactFrames_right));
%     ECy_right = zerosCol;
%     ECz_right(inContactFrames_right) = copData_filtered(inContactFrames_right,6) - ...
%         (-h*loadsForECCalc(inContactFrames_right,9) - loadsForECCalc(inContactFrames_right,10))./loadsForECCalc(inContactFrames_right,8);
%     ECz_right(outOfContactFrames_right) = median(ECz_right(outOfContactFrames_right));
%     
%     ECs = [ECx_left,ECy_left,ECz_left,ECx_right,ECy_right,ECz_right];
%     
%     ECs_mean = zeros(size(ECs));
%     ECs_stDev = zeros(size(ECs));
%     ECs_stDev(:,1:3) = ones(size(ECs,1),1)*std(ECs(inContactFrames_left,1:3));
%     ECs_stDev(:,4:6) = ones(size(ECs,1),1)*std(ECs(inContactFrames_right,4:6));
%     ECs_mean(:,1:3) = ones(size(ECs,1),1)*mean(ECs(inContactFrames_left,1:3));
%     ECs_mean(:,4:6) = ones(size(ECs,1),1)*mean(ECs(inContactFrames_right,4:6));
% else
% %     ECx_left = copData_filtered(:,1) - (loadsForECCalc(:,6))./loadsForECCalc(:,2);
% %     ECy_left = zerosCol;
% %     ECz_left = copData_filtered(:,3) - (-loadsForECCalc(:,4))./loadsForECCalc(:,2);
% %     ECx_right = copData_filtered(:,4) - (loadsForECCalc(:,12))./loadsForECCalc(:,8);
% %     ECy_right = zerosCol;
% %     ECz_right = copData_filtered(:,6) - (-loadsForECCalc(:,10))./loadsForECCalc(:,8);
%     
%     ECx_left = copData_filtered(:,1) - (h*loadsForECCalc(:,1) + loadsForECCalc(:,6))./loadsForECCalc(:,2);
%     ECy_left = zerosCol;
%     ECz_left = copData_filtered(:,3) - (-h*loadsForECCalc(:,3) - loadsForECCalc(:,4))./loadsForECCalc(:,2);
%     ECx_right = copData_filtered(:,4) - (h*loadsForECCalc(:,7) + loadsForECCalc(:,12))./loadsForECCalc(:,8);
%     ECy_right = zerosCol;
%     ECz_right = copData_filtered(:,6) - (-h*loadsForECCalc(:,9) - loadsForECCalc(:,10))./loadsForECCalc(:,8);
%     
%     ECs = [ECx_left,ECy_left,ECz_left,ECx_right,ECy_right,ECz_right];
%     
%     ECs_stDev = ones(size(ECs,1),1)*std(ECs);
%     ECs_mean = ones(size(ECs,1),1)*mean(ECs);
% end
% 
% 
% % Get rid of outliers
% ECs(abs(ECs-ECs_mean)>(ECs_stDev)) = ECs_mean(abs(ECs-ECs_mean)>(ECs_stDev)); %Remove outliers
% ECs_mean = ones(size(ECs,1),1)*mean(ECs); %Recalculate the mean without outliers

% Compute COP to compare to COP given in data
% COP1xCalc = (loadsForECCalc(:,6))./loadsForECCalc(:,2) + ECs_mean(1,1);
% COP1zCalc = (-loadsForECCalc(:,4))./loadsForECCalc(:,2) + ECs_mean(1,3);
% 
% COP2xCalc = (loadsForECCalc(:,12))./loadsForECCalc(:,8) + ECs_mean(1,4);
% COP2zCalc = (-loadsForECCalc(:,10))./loadsForECCalc(:,8) + ECs_mean(1,6);
% COP1xCalc = (h*loadsForECCalc(:,1) + loadsForECCalc(:,6))./loadsForECCalc(:,2) + ECs_mean(1,1);
% COP1zCalc = (-h*loadsForECCalc(:,3) - loadsForECCalc(:,4))./loadsForECCalc(:,2) + ECs_mean(1,3);
% 
% COP2xCalc = (h*loadsForECCalc(:,7) + loadsForECCalc(:,12))./loadsForECCalc(:,8) + ECs_mean(1,4);
% COP2zCalc = (-h*loadsForECCalc(:,9) - loadsForECCalc(:,10))./loadsForECCalc(:,8) + ECs_mean(1,6);

COP1xCalc = (h*loadsForECCalc(:,1) + loadsForECCalc(:,6))./loadsForECCalc(:,2) + ECs(1);
COP1zCalc = (-h*loadsForECCalc(:,3) - loadsForECCalc(:,4))./loadsForECCalc(:,2) + ECs(3);

COP2xCalc = (h*loadsForECCalc(:,7) + loadsForECCalc(:,12))./loadsForECCalc(:,8) + ECs(4);
COP2zCalc = (-h*loadsForECCalc(:,9) - loadsForECCalc(:,10))./loadsForECCalc(:,8) + ECs(6);

copData_calc = [COP1xCalc,zerosCol,COP1zCalc,COP2xCalc,zerosCol,COP2zCalc];

% Append electrical center info on GRF data for input into open sim
% forcePlateData_withECs = [forcePlateData_filtered ECs_mean];
forcePlateData_withECs = [forcePlateData_filtered ECs_time];

% Filter COP Data
copData_filtered = zeros(size(COPData_reordered));
if isGait
    copData_filtered(inContactFrames_left,1:3) = filterData(COPData_reordered(inContactFrames_left,1:3),1/analogSampleRate,10,4);
    copData_filtered(inContactFrames_right,4:6) = filterData(COPData_reordered(inContactFrames_right,4:6),1/analogSampleRate,10,4);
else
    copData_filtered(:,1:3) = filterData(COPData_reordered(:,1:3),1/analogSampleRate,10,4);
    copData_filtered(:,4:6) = filterData(COPData_reordered(:,4:6),1/analogSampleRate,10,4);
end


% Calculate COP and new loads based on method in treadmill user manual
%[2 3 1] means that the new [1 2 3] axis are the old [2 3 1] axis

% COP1xCalc_noOffset = (h*forcePlateData_reordered(:,1) + forcePlateData_reordered(:,6))./forcePlateData_reordered(:,2);
% COP1zCalc_noOffset = (-h*forcePlateData_reordered(:,3) - forcePlateData_reordered(:,4))./forcePlateData_reordered(:,2);
% 
% COP2xCalc_noOffset = (h*forcePlateData_reordered(:,7) + forcePlateData_reordered(:,12))./forcePlateData_reordered(:,8);
% COP2zCalc_noOffset = (-h*forcePlateData_reordered(:,9) - forcePlateData_reordered(:,10))./forcePlateData_reordered(:,8);

% copData_noOffset = [COP1xCalc_noOffset,zerosCol,COP1zCalc_noOffset,COP2xCalc_noOffset,zerosCol,COP2zCalc_noOffset];

% xOffset1 = COPData_reordered(~(isinf(COPData_reordered(:,1))|isinf(copData_noOffset(:,1))),1) - copData_noOffset(~(isinf(COPData_reordered(:,1))|isinf(copData_noOffset(:,1))),1);
% zOffset1 = COPData_reordered(~(isinf(COPData_reordered(:,3))|isinf(copData_noOffset(:,3))),3) - copData_noOffset(~(isinf(COPData_reordered(:,3))|isinf(copData_noOffset(:,3))),3);
% xOffset2 = COPData_reordered(~(isinf(COPData_reordered(:,4))|isinf(copData_noOffset(:,4))),4) - copData_noOffset(~(isinf(COPData_reordered(:,4))|isinf(copData_noOffset(:,4))),4);
% zOffset2 = COPData_reordered(~(isinf(COPData_reordered(:,6))|isinf(copData_noOffset(:,6))),6) - copData_noOffset(~(isinf(COPData_reordered(:,6))|isinf(copData_noOffset(:,6))),6);
% 
% xOffset1_mean = nanmedian(xOffset1);
% zOffset1_mean = nanmedian(zOffset1);
% xOffset2_mean = nanmedian(xOffset2);
% zOffset2_mean = nanmedian(zOffset2);

% COP1xCalc_manual = COP1xCalc_noOffset + xOffset1_mean;
% COP1zCalc_manual = COP1zCalc_noOffset + zOffset1_mean;
% COP2xCalc_manual = COP2xCalc_noOffset + xOffset2_mean;
% COP2zCalc_manual = COP2zCalc_noOffset + zOffset2_mean;

% COP1xCalc_manual = COP1xCalc_noOffset + ECs(1);
% COP1zCalc_manual = COP1zCalc_noOffset + ECs(3);
% COP2xCalc_manual = COP2xCalc_noOffset + ECs(4);
% COP2zCalc_manual = COP2zCalc_noOffset + ECs(6);
% 
% copData_manual = [COP1xCalc_manual,zerosCol,COP1zCalc_manual,COP2xCalc_manual,zerosCol,COP2zCalc_manual];
% 
figure;
plot(COPData_reordered(:,1),'k');
hold on
plot(copData_calc(:,1),'b');
% plot(copData_filtered(:,1),'r');
title('COPx 1');

figure;
plot(COPData_reordered(:,3),'k');
hold on
plot(copData_calc(:,3),'b');
% plot(copData_filtered(:,3),'r');
title('COPz 1');

figure;
plot(COPData_reordered(:,4),'k');
hold on
plot(copData_calc(:,4),'b');
% plot(copData_filtered(:,4),'r');
title('COPx 2');

figure;
plot(COPData_reordered(:,6),'k');
hold on
plot(copData_calc(:,6),'b');
% plot(copData_filtered(:,6),'r');
title('COPz 2');


figure;
plot(-loadsForECCalc(:,6),'b');
hold on
plot(loadsForECCalc(:,2),'r');
title('Loads for COPx 1');

figure;
plot(-loadsForECCalc(:,4),'b');
hold on
plot(loadsForECCalc(:,2),'r');
title('Loads for COPz 1');

figure;
plot(-loadsForECCalc(:,12),'b');
hold on
plot(loadsForECCalc(:,8),'r');
title('Loads for COPx 2');

figure;
plot(loadsForECCalc(:,10),'b');
hold on
plot(loadsForECCalc(:,8),'r');
title('Loads for COPz 2');

keyboard;

%% Parse individual steps and resample to 141 time points
if isGait
    forcePlateData_parsed = cell(numSteps,1);
    copData_parsed = cell(numSteps,1);
    markerData_parsed = cell(numSteps,1);
    emgData_parsed = cell(numSteps,1);
    rawEMGData_parsed = cell(numSteps,1);
    timeAnalog_parsed = cell(numSteps,1);
    timeMarker_parsed = cell(numSteps,1);
    
    for iii = 1:numSteps
        
        analogDataFrames = cycleBegins(iii):cycleEnds(iii);
        markerDataFrames = ceil(analogDataFrames(1:10:end)/10)+1;
        
        timeAnalog_parsed{iii} = timeAnalog(analogDataFrames);
        timeMarker_parsed{iii} = timeMarker(markerDataFrames);
        
        forcePlateData_parsed{iii} = forcePlateData_withECs(analogDataFrames,:);
        copData_parsed{iii} = copData_calc(analogDataFrames,:);
        emgData_parsed{iii} = emgData_processed(analogDataFrames,:);
        rawEMGData_parsed{iii} = EMGData(analogDataFrames,:);
        markerData_parsed{iii} = markerData_filtered(markerDataFrames,:,:);
        
    end
    
    % Resample data to 141 time points
    time_resampled = cell(numSteps,1);
    forcePlateData_resampled = cell(numSteps,1);
    copData_resampled = cell(numSteps,1);
    markerData_resampled = cell(numSteps,1);
    emgData_resampled = cell(numSteps,1);
    for iii = 1:numSteps
        timeAnalogStep = timeAnalog_parsed{iii};
        timeMarkerStep = timeMarker_parsed{iii};
        timeNew = linspace(timeMarkerStep(1), timeMarkerStep(end), 141);
        time_resampled{iii} = timeNew(:);
        
        forcePlateData_resampled{iii} = spline(timeAnalogStep,forcePlateData_parsed{iii}', timeNew)';
        copData_resampled{iii} = spline(timeAnalogStep,copData_parsed{iii}',timeNew)';
        emgData_resampled{iii} = spline(timeAnalogStep,emgData_parsed{iii}', timeNew)';
        
        for jjj = 1:sizeOfMarkerData(2)
            markerData_resampled{iii}(:,jjj,:) = spline(timeMarkerStep,squeeze(markerData_parsed{iii}(:,jjj,:))', timeNew)';
        end
        
    end
    
end


%% Assemble data into matlab structure
% time array, stim on/off times, marker data, GRF data, COP data (calculate), 
% COP data (raw), COP data (filtered), ECs, EMG
% data (raw), EMG data (processed)
if isGait
    processedData.time = time_resampled;
    if isStim
        processedData.stimOnTimes = stimOnTimes;
        processedData.stimOffTimes = stimOffTimes;
    end
    processedData.markersData = markerData_resampled;
    processedData.forcePlateData = forcePlateData_resampled;
    processedData.copData = copData_resampled;
    processedData.emgData = emgData_resampled;
    
else
    processedData.time = timeAnalog;
    if isStim
        processedData.stimOnTimes = stimOnTimes;
        processedData.stimOffTimes = stimOffTimes;
    end
    processedData.markerData = markerData_filtered;
    processedData.forcePlateData = forcePlateData_withECs;
    processedData.copData = copData_calc;
    processedData.emgData = emgData_processed;
end


%% Write data to files
folderName = strrep(fileName,'mat','');
mkdir(folderName);
originalDir = cd;
cd(folderName);

outFileMAT = strrep(fileName,'.mat','_processed.mat');
save(outFileMAT,'processedData');
if isGait
    parfor jjj = 1:numSteps

        if jjj < 10
            cycleNum = ['_0' num2str(jjj)];
        else
            cycleNum = ['_' num2str(jjj)];
        end

        outFileTRC = strrep(fileName,'.mat',['_cycle' cycleNum '.trc']);
        outFileGRF = strrep(fileName,'.mat',['_cycle' cycleNum '_GRF.mot']);
        outFileEMG = strrep(fileName,'.mat',['_cycle' cycleNum '_EMG.mot']);
        outFileRawEMG = strrep(fileName,'.mat',['_cycle' cycleNum '_rawEMG.mot']);

        outputTRCFile(outFileTRC, markerData_resampled{jjj},time_resampled{jjj},newMarkerNames);
        outputGRFmot(outFileGRF, forcePlateData_resampled{jjj},time_resampled{jjj});
        outputEMGmot(outFileEMG, emgData_resampled{jjj},time_resampled{jjj},EMGNames);
        outputEMGmot(outFileRawEMG, rawEMGData_parsed{jjj},timeAnalog_parsed{jjj}',EMGNames);
    end
    
else
    outFileTRC = strrep(fileName,'.mat','.trc');
    outFileGRF = strrep(fileName,'.mat','_GRF.mot');
    outFileEMG = strrep(fileName,'.mat','_EMG.mot');
    outFileRawEMG = strrep(fileName,'.mat','_rawEMG.mot');
    
    outputTRCFile(outFileTRC, markerData_filtered,timeMarker',newMarkerNames);
    outputGRFmot(outFileGRF, forcePlateData_withECs,timeAnalog');
    outputEMGmot(outFileEMG, emgData_processed,timeAnalog',EMGNames);
    outputEMGmot(outFileRawEMG, EMGData,timeAnalog',EMGNames);
end
cd(originalDir);


end



%% Function Definitions
function namesCells = convertCharMatrixToCells(namesCharMatrix)

numNames = size(namesCharMatrix,1);

namesCells = cell(1,numNames);
for iii = 1:numNames
    name = namesCharMatrix(iii,:);
    
    % Because a char matrix is the input, each row contains blank space at
    % the end. This removes that blank space. It assumes that if there are
    % two spaces in a row, then the rest of the line is blank.
    exitFlag = false;
    jjj = 1;
    endIndex = 1;
    while ~exitFlag
        jjj = jjj+1;
        if jjj == length(name)
            if name(jjj)==' '
                endIndex = jjj-1;
            else
                endIndex = jjj;
            end
            exitFlag = true;
        else
            if name(jjj)==' ' && name(jjj+1)==' '
                endIndex = jjj-1;
                exitFlag = true;
            end
        end
        
    end
    namesCells{1,iii} = name(1:endIndex); 
end

end

function Data = filterData(Data, dt, cutoff,degree)

if strcmp(cutoff,'VarFreq')
    cutoff = 7/(mean(dt)*101);
end

SampRate = 1/mean(dt);
[b,a] = butter(degree,cutoff/SampRate*2);
Data = filtfilt(b,a,Data);
end

function emgFiltered = processEMG(emgData, dt, HPcutoff, LPcutoff)
% Function that process EMG data according the the process described by
% Lloyd and Besier (2003)
% Inputs:
%       emgData is a M x N matrix of unfiltered EMG data where M is the
%           number of time frames and N is the number of muscles
%       dt is the time step used in the EMG data
%       HPcutoff is the cutoff frequency for the high pass filter
%       LPcutoff is the cutoff frequency for the low pass filter
% Outputs:
%       emgFiltered is the filtered EMG data

SampRate = 1/mean(dt);

% High pass filter the data
degree = 4;
[z,p,k] = butter(degree, HPcutoff/SampRate*2,'high');
[sos,g] = zp2sos(z,p,k);      % Convert to SOS(second-order-sections) form
EMG_HP = filtfilt(sos,g,emgData);

% Demean
EMG_DM = EMG_HP-ones(size(EMG_HP,1),1)*mean(EMG_HP);

% Rectify
EMG_RF = abs(EMG_DM);

% Low pass filter
degree = 4;
[z,p,k] = butter(degree,LPcutoff/SampRate*2);
[sos,g] = zp2sos(z,p,k);      % Convert to SOS(second-order-sections) form
% pad data with final values
numPad = 0;
EMG_Padded = [ones(numPad,1)*EMG_RF(1,:);EMG_RF;ones(numPad,1)*EMG_RF(end,:)];
EMG_LP = filtfilt(sos,g,EMG_Padded);
emgFiltered = EMG_LP(numPad+1:end-numPad,:);

% Remove any negative EMG values that may still exist
emgFiltered(emgFiltered<0) = 0;
end

function derivs = centralDiff(x,h)
% Calculates first derivative approximation using the central difference
% method assuming a constant step size

twoH = 2*h; %Calculate 2*h once so that it isn't recalculated every loop iteration

derivs = zeros(size(x));
derivs(1,:) = (x(2,:) - x(1,:))/h;
for iii = 2:(size(x,1)-1)
    derivs(iii,:) = (x(iii+1,:) - x(iii-1,:))/twoH;
end
derivs(end,:) = (x(end,:) - x(end-1,:))/h;

end

function stimulationIndices = getStimIndices(EMGData,sampleRate)
nPts = size(EMGData,1);

%Lowpass frequency of 6 prevents samples within the stimulated period from
%dropping below the threshhold, but does not over-smooth 
EMGSum = sum(abs(EMGData),2);
EMGCutoff_high = 0.4*max(EMGSum);
EMGCutoff_low = 0.2*max(EMGSum);
EMGSum_processed = processEMG(EMGSum,1/sampleRate,30,10); 
EMGProcessedMax = max(EMGSum_processed);
isStimulated = EMGSum_processed > 0.4*ones(nPts,1)*EMGProcessedMax(1); %Cutoff of 80% of max seems to prevent false positives but include all real peaks

currentState = 0; %Keeps track of the current state (above or below threshhold)
switchNumber = 0; %Number of switchpoints is unknown, so this keeps track how many switches there have been
for iii = 1:nPts
    if isStimulated(iii) ~= currentState
        switchNumber = switchNumber + 1;
        switchPoints(switchNumber) = iii;
        switchStates(switchNumber) = currentState;
        currentState = isStimulated(iii);
    end
end
% We need to make sure that there are an even number of switch points. If
% there are an odd number, add a new stim point at the end.
if mod(switchNumber,2)
    switchNumber = switchNumber + 1;
    switchPoints(switchNumber) = nPts;
    switchStates(switchNumber) = 1;
end

stimulationIndices = zeros(switchNumber,1);
% EMGDerivs = centralDiff(EMGSum_processed,1);
for iii = 1:switchNumber
    if switchStates(iii) %If it is switching off
%         deriv = -1;
        EMGIndex = switchPoints(iii);
        value = EMGSum(EMGIndex);
        while (value > EMGCutoff_low) && (EMGIndex < nPts)
            EMGIndex = EMGIndex + 1;
            value = EMGSum(EMGIndex);
        end
%         while (deriv < 0) && (EMGIndex < nPts)   % Depricated
%             EMGIndex = EMGIndex + 1;
%             deriv = EMGDerivs(EMGIndex);
%         end
        stimulationIndices(iii) = EMGIndex;
        
    else %If it is switching on
%         deriv = 1;
        
        EMGIndex = switchPoints(iii);
        value = EMGSum(EMGIndex);
        while (value < EMGCutoff_high) && (EMGIndex < nPts)
            EMGIndex = EMGIndex + 1;
            value = EMGSum(EMGIndex);
        end
%         while (deriv > 0) && (EMGIndex > 1)
%             EMGIndex = EMGIndex - 1;
%             deriv = EMGDerivs(EMGIndex);
%         end
        stimulationIndices(iii) = EMGIndex;
    end
end

end

function [cycleBegins,cycleEnds,heelStrikeFrames,toeOffFrames] = parseCycles(GRFVert)
GRFVert(GRFVert<0) = 0;
nPts = length(GRFVert);
isContact = GRFVert > 100;


currentState = isContact(1); %Keeps track of the current state (above or below threshhold)
switchNumber = 0; %Number of switchpoints is unknown, so this keeps track how many switches there have been
for iii = 1:nPts
    if isContact(iii) ~= currentState
        switchNumber = switchNumber + 1;
        switchPoints(switchNumber) = iii;
        switchStates(switchNumber) = currentState;
        currentState = isContact(iii);
    end
end

GRFDerivs = centralDiff(GRFVert,1);
numHeelStrikes = 0;
numToeOffs = 0;
for iii = 1:switchNumber
    deriv = GRFDerivs(switchPoints(iii));
    GRFIndex = switchPoints(iii);
    if  deriv > 0 %If heel-strike
        while (deriv > 0) && (GRFIndex > 1)
            GRFIndex = GRFIndex - 1;
            deriv = GRFDerivs(GRFIndex);
        end
        numHeelStrikes = numHeelStrikes + 1;
        heelStrikeFrames(numHeelStrikes) = GRFIndex + 2; %NOTE (Nathan): Add 2 to GRF Index bc central diff will change to zero 2 frames away from when force goes to zero
        
    elseif deriv < 0
        while (deriv < 0) && (GRFIndex < nPts)
            GRFIndex = GRFIndex + 1;
            deriv = GRFDerivs(GRFIndex);
        end
        numToeOffs = numToeOffs + 1;
        toeOffFrames(numToeOffs) = GRFIndex - 2; %NOTE (Nathan): Subtract 2 from GRF Index bc central diff will change to zero 2 frames away from when force goes to zero
    end
end

% Determine the beginnings and ends of cycles
cycleBegins = heelStrikeFrames(1:(end-1));
cycleEnds = heelStrikeFrames(2:end);

numCycleFrames = cycleEnds-cycleBegins;

% Remove final heel strike if there is not enough time following the
% it to pad the data effectively
if (heelStrikeFrames(end)+.2*numCycleFrames(end)) > numel(GRFVert)
    heelStrikeFrames(end) = [];
end
if (heelStrikeFrames(1)-.2*numCycleFrames(1)) <= 0
    heelStrikeFrames(1) = [];
end

% Determine the beginnings and ends of cycles now that some cycles
% might have been removed
cycleBegins = heelStrikeFrames(1:(end-1));
cycleEnds = heelStrikeFrames(2:end);

% Add padding at the beginnings and ends of the cycles
numCycleFrames = cycleEnds-cycleBegins;
cycleBegins = cycleBegins-.2*numCycleFrames;
cycleEnds = cycleEnds+.2*numCycleFrames;

% Round time frames up to nearest factor of 10 to maintain consistency
% between marker data and GRF data sampling rates
cycleBegins = ceil((cycleBegins-1)/10)*10;
cycleEnds = ceil((cycleEnds-1)/10)*10;

end

function [inContact_left,outOfContact_left,inContact_right,outOfContact_right] = ...
    getInOutOfContactFrames(heelStrikeFrames_left,toeOffFrames_left,heelStrikeFrames_right,toeOffFrames_right,nAnalogFrames)
% Create arrays of which frames are in contact and out of contact for
% each leg.
inContact_right = [];
outOfContact_right = [];
inContact_left = [];
outOfContact_left = [];
% Add frames between the beginning of the data and the first
% heel-stike/toe-off event
if toeOffFrames_right(1) > heelStrikeFrames_right(1)
    outOfContact_right = [outOfContact_right,1:heelStrikeFrames_right(1)];
else
    inContact_right = [inContact_right,1:toeOffFrames_right(1)];
end
if toeOffFrames_left(1) > heelStrikeFrames_left(1)
    outOfContact_left = [outOfContact_left,1:heelStrikeFrames_left(1)];
else
    inContact_left = [inContact_left,1:toeOffFrames_left(1)];
end 

% Add frames between the heel-strike and toe-off events
if length(heelStrikeFrames_right) ~= length(toeOffFrames_right)
    numRight = min([length(heelStrikeFrames_right),length(toeOffFrames_right)]);
else
    numRight = length(heelStrikeFrames_right) - 1;
end
if length(heelStrikeFrames_left) ~= length(toeOffFrames_left)
    numLeft = min([length(heelStrikeFrames_left),length(toeOffFrames_left)]);
else
    numLeft = length(heelStrikeFrames_left) - 1;
end
for iii = 1:numLeft
    if toeOffFrames_left(1) > heelStrikeFrames_left(1)
        inContact_left = [inContact_left,heelStrikeFrames_left(iii):toeOffFrames_left(iii)];
        outOfContact_left = [outOfContact_left,toeOffFrames_left(iii):heelStrikeFrames_left(iii+1)];
    else
        inContact_left = [inContact_left,heelStrikeFrames_left(iii):toeOffFrames_left(iii+1)];
        outOfContact_left = [outOfContact_left,toeOffFrames_left(iii):heelStrikeFrames_left(iii)];
    end 
end
for iii = 1:numRight
    if toeOffFrames_right(1) > heelStrikeFrames_right(1)
        inContact_right = [inContact_right,heelStrikeFrames_right(iii):toeOffFrames_right(iii)];
        outOfContact_right = [outOfContact_right,toeOffFrames_right(iii):heelStrikeFrames_right(iii+1)];
    else
        inContact_right = [inContact_right,heelStrikeFrames_right(iii):toeOffFrames_right(iii+1)];
        outOfContact_right = [outOfContact_right,toeOffFrames_right(iii):heelStrikeFrames_right(iii)];
    end
end

% Add the last set of frames between a heel-strike and a toe-off. This
% is necessary because the above loop does not include the last set.
if toeOffFrames_right(end) < heelStrikeFrames_right(end)
    inContact_right = [inContact_right,heelStrikeFrames_right(end):toeOffFrames_right(end)];
else
    outOfContact_right = [outOfContact_right,toeOffFrames_right(end):heelStrikeFrames_right(end)];
end
if toeOffFrames_left(end) < heelStrikeFrames_left(end)
    outOfContact_left = [outOfContact_left,toeOffFrames_left(end):heelStrikeFrames_left(end)];
else
    inContact_left = [inContact_left,heelStrikeFrames_left(end):toeOffFrames_left(end)];
end

% Add frames between the last heel-strike/toe-off and the end of the
% data
if toeOffFrames_right(end) < heelStrikeFrames_right(end)
    inContact_right = [inContact_right,heelStrikeFrames_right(end):nAnalogFrames];
else
    outOfContact_right = [outOfContact_right,toeOffFrames_right(end):nAnalogFrames];
end
if toeOffFrames_left(end) < heelStrikeFrames_left(end)
    inContact_left = [inContact_left,heelStrikeFrames_left(end):nAnalogFrames];
else
    outOfContact_left = [outOfContact_left,toeOffFrames_left(end):nAnalogFrames];
end 

end

function newMarkerNames = renameMarkers(markerNames)
% Renames markers from Neuromechanics Lab at Emory convention (Lina Ting's
% lab) to our convention (B.J. Fregly's lab).
numMarkers = length(markerNames);
newMarkerNames = cell(1,numMarkers);

c7Flag = false;
bkpkFlag = false;
clavFlag = false;
lshoFlag = false;
lelbFlag = false;
lwraFlag = false;
rshoFlag = false;
relbFlag = false;
rwraFlag = false;
lasiFlag = false;
rasiFlag = false;
lpsiFlag = false;
rpsiFlag = false;
lthsupFlag = false;
lthlatFlag = false;
lthinfFlag = false;
lknlatFlag = false;
lknmedFlag = false;
ltibsupFlag = false;
ltiblatFlag = false;
ltibinfFlag = false;
lanklatFlag = false;
lankmedFlag = false;
lheelFlag = false;
ltoeFlag = false;
ltjtlatFlag = false;
ltjtmedFlag = false;
lmdftsupFlag = false;
lmdftlatFlag = false;
rthsupFlag = false;
rthlatFlag = false;
rthinfFlag = false;
rknlatFlag = false;
rknmedFlag = false;
rtibsupFlag = false;
rtiblatFlag = false;
rtibinfFlag = false;
ranklatFlag = false;
rankmedFlag = false;
rheelFlag = false;
rtoeFlag = false;
rtjtlatFlag = false;
rtjtmedFlag = false;
rmdftsupFlag = false;
rmdftlatFlag = false;
lhndlFlag = false;
rhndlFlag = false;
rootFlag = false;
sacFlag = false;
for iii = 1:numMarkers
    if ~c7Flag && ~isempty(strfind(markerNames{iii},'C7'))
        newMarkerNames{iii} = 'Neck';
        c7Flag = true;
    elseif ~bkpkFlag && ~isempty(strfind(markerNames{iii},'BKPK'))
        newMarkerNames{iii} = 'Backpack';
%         newMarkerNames{iii} = 'Chest'; %Nathan: I believe these were named wrong. Check to make sure.
        bkpkFlag = true;
    elseif ~clavFlag && ~isempty(strfind(markerNames{iii},'CLAV'))
        newMarkerNames{iii} = 'Chest';
%         newMarkerNames{iii} = 'Backpack'; %Nathan: I believe these were named wrong. Check to make sure.
        clavFlag = true;
    elseif ~lshoFlag && ~isempty(strfind(markerNames{iii},'LSHO'))
        newMarkerNames{iii} = 'L.Shoulder';
        lshoFlag = true;
    elseif ~lelbFlag && ~isempty(strfind(markerNames{iii},'LELB'))
        newMarkerNames{iii} = 'L.Elbow';
        lelbFlag = true;
    elseif ~lwraFlag && ~isempty(strfind(markerNames{iii},'LWRA'))
        newMarkerNames{iii} = 'L.Wrist';
        lwraFlag = true;
    elseif ~rshoFlag && ~isempty(strfind(markerNames{iii},'RSHO'))
        newMarkerNames{iii} = 'R.Shoulder';
        rshoFlag = true;
    elseif ~relbFlag && ~isempty(strfind(markerNames{iii},'RELB'))
        newMarkerNames{iii} = 'R.Elbow';
        relbFlag = true;
    elseif ~rwraFlag && ~isempty(strfind(markerNames{iii},'RWRA'))
        newMarkerNames{iii} = 'R.Wrist';
        rwraFlag = true;
    elseif ~lasiFlag && ~isempty(strfind(markerNames{iii},'LASI'))
        newMarkerNames{iii} = 'L.ASIS';
        lasiFlag = true;
    elseif ~rasiFlag && ~isempty(strfind(markerNames{iii},'RASI'))
        newMarkerNames{iii} = 'R.ASIS';
        rasiFlag = true;
    elseif ~lpsiFlag && ~isempty(strfind(markerNames{iii},'LPSI'))
        newMarkerNames{iii} = 'L.PSIS';
        lpsiFlag = true;
    elseif ~rpsiFlag && ~isempty(strfind(markerNames{iii},'RPSI'))
        newMarkerNames{iii} = 'R.PSIS';
        rpsiFlag = true;
    elseif ~lthsupFlag && ~isempty(strfind(markerNames{iii},'LTHI_SUP'))
        newMarkerNames{iii} = 'L.Thigh.Superior';
        lthsupFlag = true;
    elseif ~lthlatFlag && ~isempty(strfind(markerNames{iii},'LTHI_LAT'))
        newMarkerNames{iii} = 'L.Thigh.Lateral';
        lthlatFlag = true;
    elseif ~lthinfFlag && ~isempty(strfind(markerNames{iii},'LTHI_INF'))
        newMarkerNames{iii} = 'L.Thigh.Inferior';
        lthinfFlag = true;
    elseif ~lknlatFlag && ~isempty(strfind(markerNames{iii},'LKNE_LAT'))
        newMarkerNames{iii} = 'L.Knee.Lateral';
        lknlatFlag = true;
    elseif ~lknmedFlag && ~isempty(strfind(markerNames{iii},'LKNE_MED'))
        newMarkerNames{iii} = 'L.Knee.Medial';
        lknmedFlag = true;
    elseif ~ltibsupFlag && ~isempty(strfind(markerNames{iii},'LTIB_SUP'))
        newMarkerNames{iii} = 'L.Shank.Superior';
        ltibsupFlag = true;
    elseif ~ltiblatFlag && ~isempty(strfind(markerNames{iii},'LTIB_LAT'))
        newMarkerNames{iii} = 'L.Shank.Lateral';
        ltiblatFlag = true;
    elseif ~ltibinfFlag && ~isempty(strfind(markerNames{iii},'LTIB_INF'))
        newMarkerNames{iii} = 'L.Shank.Inferior';
        ltibinfFlag = true;
    elseif ~lanklatFlag && ~isempty(strfind(markerNames{iii},'LANK_LAT'))
        newMarkerNames{iii} = 'L.Ankle.Lateral';
        lanklatFlag = true;
    elseif ~lankmedFlag && ~isempty(strfind(markerNames{iii},'LANK_MED'))
        newMarkerNames{iii} = 'L.Ankle.Medial';
        lankmedFlag = true;
    elseif ~lheelFlag && ~isempty(strfind(markerNames{iii},'LHEE'))
        newMarkerNames{iii} = 'L.Heel';
        lheelFlag = true;
    elseif ~ltoeFlag && ~isempty(strfind(markerNames{iii},'LTOE'))
        newMarkerNames{iii} = 'L.Toe';
        ltoeFlag = true;
    elseif ~ltjtlatFlag && ~isempty(strfind(markerNames{iii},'LTJT_LAT'))
        newMarkerNames{iii} = 'L.Toe.Lateral';
        ltjtlatFlag = true;
    elseif ~ltjtmedFlag && ~isempty(strfind(markerNames{iii},'LTJT_MED'))
        newMarkerNames{iii} = 'L.Toe.Medial';
        ltjtmedFlag = true;
    elseif ~lmdftsupFlag && ~isempty(strfind(markerNames{iii},'LMIDFOOT_SUP'))
        newMarkerNames{iii} = 'L.Midfoot.Superior';
        lmdftsupFlag = true;
    elseif ~lmdftlatFlag && ~isempty(strfind(markerNames{iii},'LMIDFOOT_LAT'))
        newMarkerNames{iii} = 'L.Midfoot.Lateral';
        lmdftlatFlag = true;
    elseif ~rthsupFlag && ~isempty(strfind(markerNames{iii},'RTHI_SUP'))
        newMarkerNames{iii} = 'R.Thigh.Superior';
        rthsupFlag = true;
    elseif ~rthlatFlag && ~isempty(strfind(markerNames{iii},'RTHI_LAT'))
        newMarkerNames{iii} = 'R.Thigh.Lateral';
        rthlatFlag = true;
    elseif ~rthinfFlag && ~isempty(strfind(markerNames{iii},'RTHI_INF'))
        newMarkerNames{iii} = 'R.Thigh.Inferior';
        rthinfFlag = true;
    elseif ~rknlatFlag && ~isempty(strfind(markerNames{iii},'RKNE_LAT'))
        newMarkerNames{iii} = 'R.Knee.Lateral';
        rknlatFlag = true;
    elseif ~rknmedFlag && ~isempty(strfind(markerNames{iii},'RKNE_MED'))
        newMarkerNames{iii} = 'R.Knee.Medial';
        rknmedFlag = true;
    elseif ~rtibsupFlag && ~isempty(strfind(markerNames{iii},'RTIB_SUP'))
        newMarkerNames{iii} = 'R.Shank.Superior';
        rtibsupFlag = true;
    elseif ~rtiblatFlag && ~isempty(strfind(markerNames{iii},'RTIB_LAT'))
        newMarkerNames{iii} = 'R.Shank.Lateral';
        rtiblatFlag = true;
    elseif ~rtibinfFlag && ~isempty(strfind(markerNames{iii},'RTIB_INF'))
        newMarkerNames{iii} = 'R.Shank.Inferior';
        rtibinfFlag = true;
    elseif ~ranklatFlag && ~isempty(strfind(markerNames{iii},'RANK_LAT'))
        newMarkerNames{iii} = 'R.Ankle.Lateral';
        ranklatFlag = true;
    elseif ~rankmedFlag && ~isempty(strfind(markerNames{iii},'RANK_MED'))
        newMarkerNames{iii} = 'R.Ankle.Medial';
        rankmedFlag = true;
    elseif ~rheelFlag && ~isempty(strfind(markerNames{iii},'RHEE'))
        newMarkerNames{iii} = 'R.Heel';
        rheelFlag = true;
    elseif ~rtoeFlag && ~isempty(strfind(markerNames{iii},'RTOE'))
        newMarkerNames{iii} = 'R.Toe';
        rtoeFlag = true;
    elseif ~rtjtlatFlag && ~isempty(strfind(markerNames{iii},'RTJT_LAT'))
        newMarkerNames{iii} = 'R.Toe.Lateral';
        rtjtlatFlag = true;
    elseif ~rtjtmedFlag && ~isempty(strfind(markerNames{iii},'RTJT_MED'))
        newMarkerNames{iii} = 'R.Toe.Medial';
        rtjtmedFlag = true;
    elseif ~rmdftsupFlag && ~isempty(strfind(markerNames{iii},'RMIDFOOT_SUP'))
        newMarkerNames{iii} = 'R.Midfoot.Superior';
        rmdftsupFlag = true;
    elseif ~rmdftlatFlag && ~isempty(strfind(markerNames{iii},'RMIDFOOT_LAT'))
        newMarkerNames{iii} = 'R.Midfoot.Lateral';
        rmdftlatFlag = true;
    elseif ~lhndlFlag && ~isempty(strfind(markerNames{iii},'L_handle'))
        newMarkerNames{iii} = 'L.Handle';
        lhndlFlag = true;
    elseif ~rhndlFlag && ~isempty(strfind(markerNames{iii},'R_handle'))
        newMarkerNames{iii} = 'R.Handle';
        rhndlFlag = true;
    elseif ~rootFlag && ~isempty(strfind(markerNames{iii},'Root'))
        newMarkerNames{iii} = 'Back';
        rootFlag = true;
    elseif ~sacFlag && ~isempty(strfind(markerNames{iii},'SAC'))
        newMarkerNames{iii} = 'Sacral';
        sacFlag = true;
    
    else
        newMarkerNames{iii} = markerNames{iii};
    end
end

end

function splinedMarkers = splineDroppedMarkers(markerData)

% Find which markers drop out and when
nMarkers = size(markerData,2);
nFrames = size(markerData,1);
isMarkerDropped = zeros(nFrames,nMarkers);
% If all three marker coordinates fall below a set value, we assume the
% marker has dropped
for iii = 1:nMarkers
    isMarkerDropped(:,iii) = (abs(markerData(:,iii,1))<1e-4) && (abs(markerData(:,iii,2))<1e-4) && (abs(markerData(:,iii,3))<1e-4);
end

% Locate begining and end of each drop
currentState = 0; %Keeps track of the current state (1 = marker dropped)
switchNumber = 0; %Number of switchpoints is unknown, so this keeps track how many switches there have been
for iii = 1:nFrames
    if isMarkerDropeed(iii) ~= currentState
        switchNumber = switchNumber + 1;
        switchPoints(switchNumber) = iii;
        currentState = isStimulated(iii);
    end
end

if switchNumber % If there are marker drops
    % We need to make sure that there are an even number of switch points. If
    % there are an odd number, add a new stim point at the end.
    if mod(switchNumber,2)
        switchNumber = switchNumber + 1;
        switchPoints(switchNumber) = nFrames;
    end
    droppedStart = switchPoints(1:2:end-1);
    droppedEnd = switchPoints(2:2:end);
    
    % Spline to get the dropped frames
    if droppedStart(1) == 1
        
    end
    
else % If there are no marker drops
    splinedMarkers = markerData;
end

end

function outputGRFmot(outFile,GRFData,Time)

Data = [Time GRFData];

Header = [outFile sprintf('\n') ...
    'version=1' sprintf('\n') ...
    'nRows=' num2str(size(Data,1)) sprintf('\n') ...
    'nColumns=' num2str(size(Data,2)) sprintf('\n') ...
    'inDegrees=no' sprintf('\n') ...
    'endheader' sprintf('\n') ...
    'time' sprintf('\t') 'F1x' sprintf('\t') 'F1y' sprintf('\t') 'F1z' sprintf('\t') 'M1x' sprintf('\t')...
    'M1y' sprintf('\t') 'M1z' sprintf('\t') 'F2x' sprintf('\t') 'F2y' sprintf('\t') 'F2z' sprintf('\t')...
    'M2x' sprintf('\t') 'M2y' sprintf('\t') 'M2z' sprintf('\t') 'EC1x' sprintf('\t')...
    'EC1y' sprintf('\t') 'EC1z' sprintf('\t') 'EC2x' sprintf('\t')...
    'EC2y' sprintf('\t') 'EC2z' sprintf('\n')];

nRows = size(Data,1);
nColumns = size(Data,2);

DataString = '';
for j = 1:nRows
    for k = 1:nColumns
        if k ~= nColumns
            DataString = [DataString num2str(Data(j,k)) sprintf('\t')];
        else
            DataString = [DataString num2str(Data(j,k)) sprintf('\n')];
        end
    end
end

fid = fopen(outFile,'w');
fprintf(fid,'%s',[Header DataString]);
fclose(fid);
end

function outputEMGmot(outFile,EMGData,Time, EMGnames)

Data = [Time EMGData];

Header = [outFile sprintf('\n') ...
    'version=1' sprintf('\n') ...
    'nRows=' num2str(size(Data,1)) sprintf('\n') ...
    'nColumns=' num2str(size(Data,2)) sprintf('\n') ...
    'inDegrees=no' sprintf('\n') ...
    'endheader' sprintf('\n') ...
    'time' sprintf('\t')];

for i = 1:length(EMGnames)
    if i ~= length(EMGnames)
        Header = [Header EMGnames{i} sprintf('\t')];
    else
        Header = [Header EMGnames{i} sprintf('\n')];
    end
end

nRows = size(Data,1);
nColumns = size(Data,2);
DataString = '';
for j = 1:nRows
    for k = 1:nColumns
        if k ~= nColumns
            DataString = [DataString num2str(Data(j,k)) sprintf('\t')];
        else
            DataString = [DataString num2str(Data(j,k)) sprintf('\n')];
        end
    end
end

fid = fopen(outFile,'w');
fprintf(fid,'%s',[Header DataString]);
fclose(fid);
end

function outputTRCFile(outFile, MarkerData,TimeVector,markerNames)

nMarkers = length(markerNames);

Header = HeaderTRC;

rate = 1/mean(diff(TimeVector));
AllMarkersTRC = MarkerData;

MarkerLabels = markerNames;
Header = strrep(Header,'DATARATE_INS',sprintf('%.12f',rate));
Header = strrep(Header,'CAMRATE_INS',sprintf('%.12f',rate));
Header = strrep(Header,'ORIGRATE_INS',sprintf('%.12f',rate));
Header = strrep(Header,'NFRAMES_INS',sprintf('%d',length(TimeVector)));
Header = strrep(Header,'NMARKERS_INS',sprintf('%d',nMarkers));
Header = strrep(Header,'UNITS_INS','m');
Header = strrep(Header,'FILENAME',outFile);

% Create Marker Labels String
RemoveXYZLabel = 0;
if length(MarkerLabels)==(nMarkers*3)
    MarkerLabels=MarkerLabels([1:3:end]);
    RemoveXYZLabel = 1;
end
MarkerLabelString ='';
XYZString ='';
for m=1:nMarkers
    if length(MarkerLabels)<m
        label = ['Marker' num2str(m)];
    else
        label = MarkerLabels{m};
        if RemoveXYZLabel
            label = strrep(label,'_x','');
        end
    end
    XYZString = [XYZString sprintf('X%d\tY%d\tZ%d\t',m,m,m)];
    MarkerLabelString = [MarkerLabelString sprintf('%s\t\t\t',label)];
end
Header = strrep(Header,'FIRSTMARKERNAME_INS',MarkerLabelString);
Header = strrep(Header,'XYZ_INS',XYZString);

% Create Data string
DataString = '';
for f = 1:length(TimeVector)
    DataString = [DataString sprintf('%d\t%.12f\t',f,TimeVector(f))];
    for m=1:nMarkers
        DataString = [DataString sprintf('%.12f\t%.12f\t%.12f\t',AllMarkersTRC(f,m,1),AllMarkersTRC(f,m,2),AllMarkersTRC(f,m,3))];
    end
    DataString = [DataString sprintf('\n')];
end
Header = strrep(Header,'DATASTART_INS',DataString);

fid = fopen(outFile,'w');
fprintf(fid,'%s',Header);
fclose(fid);
end

function header = HeaderTRC

header = ['PathFileType	4	(X/Y/Z)	FILENAME.trc' sprintf('\n') ...
    'DataRate	CameraRate	NumFrames	NumMarkers	Units	OrigDataRate	OrigDataStartFrame	OrigNumFrames' sprintf('\n') ...
    'DATARATE_INS	CAMRATE_INS	NFRAMES_INS	NMARKERS_INS	UNITS_INS	ORIGRATE_INS	1	       NFRAMES_INS' sprintf('\n') ...
    'Frame#	Time	FIRSTMARKERNAME_INS' sprintf('\n') ...
    sprintf('\t')  sprintf('\t')	'XYZ_INS' sprintf('\n') ...
    '' sprintf('\n') ...
    'DATASTART_INS' sprintf('\n')];
end

